from pathlib import Path

from readmeai.config.settings import ConfigLoader
from readmeai.generators.quickstart import QuickStartGenerator
from readmeai.ingestion.file_processor import FileProcessor
from readmeai.ingestion.metadata_extractor import MetadataExtractor
from readmeai.ingestion.models import RepositoryContext


class RepositoryProcessor:
    """
    Processes a repository to extract dependencies and metadata.
    """

    def __init__(self, config: ConfigLoader):
        self.config = config
        self.file_processor = FileProcessor(config)
        self.metadata_extractor = MetadataExtractor(config)
        self.quickstart_generator = QuickStartGenerator(config)

    async def process_repository(
        self, repo_path: Path | str | None = None
    ) -> RepositoryContext:
        """Process the repository and extract metadata."""
        repo_path = Path(str(repo_path))

        file_contexts = self.file_processor.process_files(repo_path)

        metadata = self.metadata_extractor.extract_metadata(file_contexts)

        language_counts = self.file_processor.count_languages(file_contexts)

        dependencies = self.file_processor.extract_dependencies(file_contexts)

        language_names = list(
            {file.language for file in file_contexts if file.language}
        )
        quickstart = self.quickstart_generator.generate(
            language_counts, metadata
        )
        dependencies_and_tools = (
            [tool for tool_group in metadata.values() for tool in tool_group]
            + language_names
            + dependencies
        )
        return RepositoryContext(
            files=file_contexts,
            dependencies=dependencies_and_tools,
            languages=language_names,
            language_counts=language_counts,
            metadata=metadata,
            quickstart=quickstart,
        )


'''
import fnmatch
import os
from collections.abc import Generator
from pathlib import Path
from typing import Dict, List, Union

from readmeai.config.settings import ConfigLoader
from readmeai.ingestion.models import FileContext, RepositoryContext
from readmeai.logger import get_logger
from readmeai.parsers.factory import ParserFactory
from readmeai.preprocessor.document_cleaner import DocumentCleaner
from readmeai.preprocessor.file_filter import is_excluded

_logger = get_logger(__name__)


class RepositoryProcessor:
    """
    Processes a repository to extract dependencies and metadata.
    """

    def __init__(self, config: ConfigLoader):
        self.document_cleaner = DocumentCleaner()
        self.cicd = config.tooling.get("cicd", {})
        self.commands = config.commands.get("quickstart_guide", {})
        self.containers = config.tooling.get("containers", {})
        self.documentation = config.tooling.get("documentation", {})
        self.language_names = config.languages.get("language_names")
        self.ignore_list = config.ignore_list.get("ignore_list")
        self.package_managers = config.tooling.get("package_managers", {})
        self.parser_files = config.parsers.get("parsers", {})

    def build_file_context(
        self, repo_path: Path
    ) -> Generator[FileContext, None, None]:
        """
        Generate file info for the given repository path.
        """
        for file_path in repo_path.rglob("*"):
            if file_path.is_file() and not is_excluded(
                self.ignore_list,
                file_path,
                repo_path,
            ):
                _logger.debug(f"Processing file: {file_path}")
                yield self._create_file_context(file_path, repo_path)
            else:
                _logger.debug(f"Skipping file: {file_path}")

    def _create_file_context(
        self, file_path: Path, repo_path: Path
    ) -> FileContext:
        """
        Create a file context object for the given file path.
        """
        file_name = file_path.name
        file_ext = file_path.suffix.lstrip(".")
        file_format = self._map_language(file_ext)
        content = file_path.read_text(errors="ignore")
        cleaned_content = self.document_cleaner.clean(content)
        relative_path = str(file_path.relative_to(repo_path))

        try:
            dependencies = self.extract_dependencies(
                [
                    FileContext(
                        path=relative_path,
                        content=content,
                        name=file_name,
                        ext=file_ext,
                        language=file_format,
                        dependencies=[],
                    )
                ]
            )
        except Exception as e:
            _logger.error(
                f"Error extracting dependencies for file {file_path}: {e!s}"
            )
            dependencies = []

        return FileContext(
            path=relative_path,
            name=file_name,
            ext=file_ext,
            content=cleaned_content,
            language=file_format,
            dependencies=dependencies,
        )

    async def process_repository(
        self,
        repo_path: Union[Path, str, None] = None,
        repo_url: Union[str, None] = None,
        temp_dir: Union[str, None] = None,
    ) -> RepositoryContext:
        """
        Process the local repository contents.
        """
        file_contexts = list(self.build_file_context(Path(str(repo_path))))
        dependencies = set().union(
            *(file.dependencies for file in file_contexts)
        )
        languages = list(
            {file.language for file in file_contexts if file.language}
        )
        cicd = self.detect_cicd_tools(file_contexts)
        containers = self.detect_tools(file_contexts, self.containers)
        documentation = self.detect_documentation_tools(file_contexts)
        package_managers = self.detect_tools(
            file_contexts, self.package_managers
        )

        metadata = {
            "cicd": cicd,
            "containers": containers,
            "documentation": documentation,
            "package_managers": package_managers,
        }

        deps = list(
            set(
                [
                    dependency
                    for metadata in metadata.values()
                    for dependency in metadata
                ]
                + languages
                + list(dependencies)
            )
        )

        language_counts: Dict[str, int] = {}
        for file in file_contexts:
            if file.ext:
                language_counts[file.ext] = (
                    language_counts.get(file.ext, 0) + 1
                )
        language_counts = dict(
            sorted(
                language_counts.items(), key=lambda item: item[1], reverse=True
            )
        )

        return RepositoryContext(
            files=file_contexts,
            dependencies=list(set(deps)),
            languages=languages,
            language_counts=language_counts,
            metadata=metadata,
        )

    def extract_dependencies(self, files: List[FileContext]) -> List[str]:
        """
        Extract dependencies from the given list of files.
        """
        dependencies = []

        for file in files:
            parser = ParserFactory.create_parser(file.name)

            if parser:
                _logger.info(
                    f"Using {parser.__class__.__name__} parser for: {file.name}"
                )
                try:
                    parsed_deps = parser.parse(file.content)
                    if parsed_deps is not None:
                        dependencies.extend(parsed_deps)
                    else:
                        _logger.warning(
                            f"Parser returned None for file: {file.name}"
                        )
                except Exception as e:
                    _logger.error(
                        f"Error parsing dependency file {file.name}: {e!s}"
                    )
            else:
                _logger.info(f"No parser found for file: {file.name}")

        return dependencies

    def _match_file_pattern(self, file_path: str, pattern: str) -> bool:
        if pattern.endswith("*"):
            pattern_parts = pattern.rstrip("*").split("/")
            file_parts = file_path.split(os.path.sep)
            return (
                len(file_parts) >= len(pattern_parts)
                and file_parts[: len(pattern_parts)] == pattern_parts
            )
        elif pattern.startswith("*"):
            return file_path.endswith(pattern[1:])
        elif "*" in pattern:
            return self._match_glob_pattern(file_path, pattern)
        return file_path.endswith(pattern)

    def _match_glob_pattern(self, file_path: str, pattern: str) -> bool:
        return fnmatch.fnmatch(file_path, pattern)

    def detect_tools(
        self, files: List[FileContext], tool_definitions: Dict[str, List[str]]
    ) -> Dict[str, str]:
        detected_tools = {}
        for file in files:
            for tool, file_patterns in tool_definitions.items():
                for pattern in file_patterns:
                    if self._match_file_pattern(file.path, pattern):
                        _logger.debug(
                            f"Matched {tool} with pattern {pattern} for file {file.path}"
                        )
                        detected_tools[tool] = file.path
                    else:
                        _logger.debug(
                            f"No match for {tool} with pattern {pattern} for file {file.path}"
                        )
        return detected_tools

    def detect_cicd_tools(self, files: List[FileContext]) -> Dict[str, str]:
        return self.detect_tools(files, self.cicd)

    def detect_documentation_tools(
        self, files: List[FileContext]
    ) -> Dict[str, str]:
        return self.detect_tools(files, self.documentation)

    def _map_language(self, file_ext: str) -> str:
        """
        Map the file extension to the programming language name.
        """
        return self.language_names.get(
            file_ext.lstrip("."), self.language_names.get("default")
        )
'''



import xml.etree.ElementTree as ET

from readmeai.logger import get_logger

_logger = get_logger(__name__)


def convert_svg_to_html(title: str, output_svg_path: str) -> str:
    """Generate a stylish banner for the README file."""
    svg_content = f"""
    <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 800 200">
        <defs>
            <linearGradient id="bg-gradient" x1="0%" y1="0%" x2="100%" y2="100%">
                <stop offset="0%" style="stop-color:#4158D0;stop-opacity:1" />
                <stop offset="50%" style="stop-color:#C850C0;stop-opacity:1" />
                <stop offset="100%" style="stop-color:#FFCC70;stop-opacity:1" />
            </linearGradient>
            <filter id="shadow">
                <feDropShadow dx="0" dy="4" stdDeviation="4" flood-opacity="0.25" />
            </filter>
        </defs>
        <rect width="800" height="200" fill="url(#bg-gradient)" rx="15" ry="15"/>
        <text x="400" y="100" font-family="Arial, sans-serif" font-size="48"
        font-weight="bold" text-anchor="middle" dominant-baseline="middle"
        fill="#FFFFFF" filter="url(#shadow)">{title.upper()}</text>
    </svg>
    """

    root = ET.fromstring(svg_content)
    tree = ET.ElementTree(root)
    tree.write(output_svg_path, encoding="utf-8", xml_declaration=True)

    html_content = f"""
    <p align="center">
      <img src="{output_svg_path}" alt="{title.lower()}-banner" width="800">
    </p>
    """

    _logger.info(f"HTML content to embed in README.md: {html_content}")

    return output_svg_path



import enum
from typing import Any, ClassVar

from readmeai.templates.base import BaseTemplate


class HeaderStyleOptions(str, enum.Enum):
    """
    Enum of supported 'Header' template styles for the README file.
    """

    CLASSIC = "classic"
    COMPACT = "compact"
    MODERN = "modern"


class HeaderTemplate(BaseTemplate):
    """
    Class variable for rendering the README.md header style.
    """

    HEADER_TEMPLATES: ClassVar[dict] = {
        HeaderStyleOptions.CLASSIC: """\
<p align="{align}">
  <img src="{image}" width="{image_width}" alt="{repo_name}-logo">
</p>
<p align="{align}">
    <h1 align="{align}">{repo_name}</h1>
</p>
<p align="{align}">
    <em>{slogan}</em>
</p>
<p align="{align}">\n\t{shields_icons}</p>
<p align="{align}">\n\t{badges_tech_stack}</p>
<br>
""",
        HeaderStyleOptions.COMPACT: """\
[<img src="{image}" align="left" width="{image_width}" padding="20">]()

## &nbsp;&nbsp; {repo_name}

&nbsp;&nbsp;&nbsp;&nbsp; *{slogan}*

<p align="left">&nbsp;&nbsp;\n\t{shields_icons}</p>
<br>
""",
        HeaderStyleOptions.MODERN.value: """\
[<img src="{image}" align="right" width="25%" padding-right="350">]()

# `{repo_name}`

#### {slogan}

<p align="{align}">\n\t{shields_icons}</p>
<p align="{align}">\n\t{badges_tech_stack}</p>
<br>
""",
    }

    def __init__(self, style: str = HeaderStyleOptions.CLASSIC) -> None:
        self.style = style

    def render(self, data: dict[str, Any]) -> str:
        """Render the header based on the provided data."""
        template = self.HEADER_TEMPLATES.get(
            self.style,
            self.HEADER_TEMPLATES[HeaderStyleOptions.CLASSIC],
        )
        return template.format(**data)

    @staticmethod
    def get_header_template(template: str) -> str:
        """Get the header template for the given style."""
        return HeaderTemplate.HEADER_TEMPLATES[HeaderStyleOptions(template)]



"""Google Gemini LLM API service implementation."""

import os
from typing import Any

import aiohttp
import google.generativeai as genai
from tenacity import (
    retry,
    retry_if_exception_type,
    stop_after_attempt,
    wait_exponential,
)

from readmeai.config.settings import ConfigLoader
from readmeai.ingestion.models import RepositoryContext
from readmeai.models.base import BaseModelHandler
from readmeai.models.tokens import token_handler


class GeminiHandler(BaseModelHandler):
    """
    Google Gemini LLM API service implementation.
    """

    def __init__(
        self, config_loader: ConfigLoader, context: RepositoryContext
    ) -> None:
        super().__init__(config_loader, context)
        self._model_settings()

    def _model_settings(self):
        api_key = os.getenv("GOOGLE_API_KEY")
        genai.configure(api_key=api_key)
        self.model_name = "gemini-1.5-flash"
        self.model = genai.GenerativeModel(self.model_name)
        self.top_p = self.config.llm.top_p

    async def _build_payload(self, prompt: str, tokens: int) -> dict[str, Any]:
        """Build payload for POST request to the Gemini API."""
        return genai.types.GenerationConfig(
            max_output_tokens=self.max_tokens,
            temperature=self.temperature,
            top_p=self.top_p,
        )

    @retry(
        stop=stop_after_attempt(3),
        wait=wait_exponential(multiplier=1, min=4, max=10),
        retry=retry_if_exception_type(
            (
                aiohttp.ClientError,
                aiohttp.ClientResponseError,
                aiohttp.ClientConnectorError,
            ),
        ),
    )
    async def _make_request(
        self,
        index: str | None,
        prompt: str | None,
        tokens: int | None,
        repo_files: Any,
    ) -> Any:
        """Processes Gemini API responses and returns generated text."""
        try:
            prompt = await token_handler(self.config, index, prompt, tokens)

            parameters = await self._build_payload(prompt, tokens)

            async with self.rate_limit_semaphore:
                response = await self.model.generate_content_async(
                    prompt,
                    generation_config=parameters,
                )
                data = response.text
                self._logger.info(
                    f"Response from Gemini for '{index}': {data}"
                )
                return index, data

        except (
            aiohttp.ClientError,
            aiohttp.ClientResponseError,
            aiohttp.ClientConnectorError,
        ) as e:
            self._logger.error(
                f"Error processing request for '{index}': {e!r}"
            )
            raise  # Re-raise for retry decorator

        except Exception as e:
            self._logger.error(f"Unexpected error for '{index}': {e!r}")
            return index, self.placeholder
